<?php

/**
 * @file
 * Contains \Drupal\yamlform\Controller\YamlFormResultsExportController.
 */

namespace Drupal\yamlform\Controller;

use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\yamlform\Entity\YamlFormSubmission;
use Drupal\yamlform\Utility\YamlFormEntityReferenceHelper;
use Drupal\yamlform\Utility\YamlFormOptionsHelper;
use Drupal\yamlform\YamlFormInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Controller routines for YAML form submission export.
 */
class YamlFormResultsExportController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Default options for exporting a CSV.
   *
   * @var array
   */
  static public $defaultExportOptions = [
    'delimiter' => ',',
    'header_keys' => 'label',
    'options_format' => 'compact',
    'options_item_format' => 'label',
    'entity_reference_format' => 'link',
    'limit' => 0,
    'sid_start' => '',
    'sid_end' => '',
    'download' => TRUE,
  ];

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The query factory to create entity queries.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory
   */
  public $queryFactory;

  /**
   * Constructs a YamlFormResultsExportController object.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   * @param \Drupal\Core\Entity\Query\QueryFactory $query_factory
   *   The query object that can query the given entity type.
   */
  public function __construct(FormBuilderInterface $form_builder, EntityManagerInterface $entity_manager, QueryFactory $query_factory) {
    $this->formBuilder = $form_builder;
    $this->entityManager = $entity_manager;
    $this->queryFactory = $query_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('entity.manager'),
      $container->get('entity.query')
    );
  }

  /**
   * Returns YAML form submission as a CSV.
   */
  public function index(Request $request, YamlFormInterface $yamlform) {
    $query = $request->query->all();
    unset($query['destination']);
    if ($query) {
      return $this->download($yamlform, $query + self::$defaultExportOptions);
    }
    else {
      return $this->formBuilder->getForm('Drupal\yamlform\Form\YamlFormResultsExportForm', $yamlform);
    }
  }

  /**
   * Download YAML form submission as a CSV.
   *
   * From: http://obtao.com/blog/2013/12/export-data-to-a-csv-file-with-symfony/
   *
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   A YAML form.
   * @param array $export_options
   *   An associative array of export options generated via the
   *   Drupal\yamlform\Form\YamlFormResultsExportForm.
   *
   * @return \Symfony\Component\HttpFoundation\StreamedResponse
   *   A streamed response that generates a CSV.
   */
  protected function download(YamlFormInterface $yamlform, array $export_options) {

    /** @var \Drupal\yamlform\YamlFormSubmissionViewBuilder $view_builder */
    $view_builder = $this->entityManager->getViewBuilder('yamlform_submission');
    $response = new StreamedResponse(function () use ($yamlform, $export_options, $view_builder) {
      $query = $this->queryFactory->get('yamlform_submission')
        ->condition('yamlform_id', $yamlform->id())
        ->sort('sid');

      if ($export_options['sid_start']) {
        $query->condition('sid', $export_options['sid_start'], '>=');
      }
      if ($export_options['sid_end']) {
        $query->condition('sid', $export_options['sid_end'], '<=');
      }
      if ($export_options['limit']) {
        $query->range(0, $export_options['limit']);
      }

      $entity_ids = $query->execute();
      if (empty($entity_ids)) {
        return;
      }

      // TODO: Consider preloading all entity references to improve performance.
      $yamlform_submissions = YamlFormSubmission::loadMultiple($entity_ids);
      $columns = $yamlform->getInputColumns();

      // Build headers.
      $header = [
        'sid',
        'created',
        'changed',
        'uid',
        'remote_addr',
      ];
      if ($export_options['header_keys'] == 'label') {
        /** @var \Drupal\Core\Field\BaseFieldDefinition[] $definitions */
        $definitions = $this->entityManager('yamlform_submission')->getBaseFieldDefinitions('yamlform_submission');
        foreach ($header as $index => $field_name) {
          $header[$index] = $definitions[$field_name]->getLabel();
        }
      }
      self::formatHeaderEntity($header, 'Entity', 'entity', $export_options);
      foreach ($columns as $column) {
        if ($export_options['options_format'] == 'separate' && isset($column['element']['#options'])) {
          $element_options = $column['element']['#options'];
          foreach ($element_options as $option_value => $option_text) {
            $header[] = ($export_options['options_item_format'] == 'key') ? $option_value : $option_text;
          }
        }
        elseif ($column['element']['#type'] == 'entity_autocomplete' && empty($column['element']['#tags'])) {
          self::formatHeaderEntity($header, $column['title'], $column['name'], $export_options);
        }
        else {
          $header[] = ($export_options['header_keys'] == 'label') ? $column['title'] : $column['name'];
        }
      }

      $handle = fopen('php://output', 'r+');

      // Build records.
      fputcsv($handle, $header, $export_options['delimiter']);
      foreach ($yamlform_submissions as $yamlform_submission) {
        $record = [];

        $record[] = $yamlform_submission->id();
        $record[] = date('c', $yamlform_submission->created->value);
        $record[] = date('c', $yamlform_submission->changed->value);
        $record[] = $yamlform_submission->uid->target_id;
        $record[] = $yamlform_submission->remote_addr->value;
        if ($entity = $yamlform_submission->getSourceEntity()) {
          $record[] = $entity->id();
          $record[] = $entity->label();
          $record[] = $entity->toUrl('canonical', ['absolute' => TRUE])->toString();
        }
        else {
          $record[] = '';
          $record[] = '';
          $record[] = '';
        }

        $data = $yamlform_submission->getData();
        foreach ($columns as $column_name => $column) {
          $column_name = $column['name'];
          $value = (isset($data[$column_name])) ? $data[$column_name] : '';

          // Handle select, radios and checkbox options.
          if (isset($column['element']['#options'])) {
            YamlFormResultsExportController::formatRecordOptions($record, $value, $column, $export_options);
          }
          elseif ($column['element']['#type'] == 'entity_autocomplete') {
            YamlFormResultsExportController::formatRecordEntityAutocomplete($record, $value, $column, $export_options);
          }
          else {
            $value = $view_builder->formatElementValueAsText($column['element'], $value);
            $record[] = $value;
          }

        }

        fputcsv($handle, $record, $export_options['delimiter']);
      }

      fclose($handle);
      return;
    });

    if ($export_options['download']) {
      $response->headers->set('Content-Type', 'application/force-download');
      $response->headers->set('Content-Disposition', 'attachment; filename="' . $yamlform->id() . '.csv"');
    }
    else {
      $response->headers->set('Content-Type', 'text/plain; charset=utf-8');
    }

    return $response;
  }

  /**
   * Format exported submission's record options.
   *
   * @param array $record
   *   Exported submission's record.
   * @param mixed $value
   *   The submitted value.
   * @param array $column
   *   The column/element for the submitted value.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordOptions(array &$record, $value, array $column, array $export_options) {
    $element_options = $column['element']['#options'];
    if ($export_options['options_format'] == 'separate') {
      // Separate multiple values (ie options).
      foreach ($element_options as $option_value => $option_text) {
        if (is_array($value) && isset($value[$option_value])) {
          $record[] = 'X';
        }
        elseif ($value == $option_value) {
          $record[] = 'X';
        }
        else {
          $record[] = '';
        }
      }
    }
    else {
      // Handle multiple values with options.
      if (is_array($value)) {
        if ($export_options['options_item_format'] == 'label') {
          $value = YamlFormOptionsHelper::getOptionsText($value, $element_options);
        }
        $record[] = implode(',', $value);
      }
      // Handle single values with options.
      elseif ($export_options['options_item_format'] == 'label') {
        $record[] = YamlFormOptionsHelper::getOptionText($value, $element_options);
      }
    }
  }

  /**
   * Format exported submission's record entity (reference) autocomplete.
   *
   * @param array $record
   *   Exported submission's record.
   * @param mixed $value
   *   The submitted value.
   * @param array $column
   *   The column/element for the submitted value.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordEntityAutocomplete(&$record, $value, $column, $export_options) {
    $target_type = $column['element']['#target_type'];
    if (!empty($column['element']['#tags'])) {
      $record[] = YamlFormEntityReferenceHelper::formatTargetEntities($target_type, $value, FALSE);
    }
    else {
      self::formatRecordEntity($record, $target_type, $value, $value, $export_options);
    }
  }

  /**
   * Format entity as a record.
   *
   * @param array $record
   *   Exported submission's record.
   * @param string $entity_type
   *   The entity type.
   * @param string|int $entity_id
   *   The entity value.
   * @param string|int $value
   *   Value to be displayed if entity is not available.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordEntity(array &$record, $entity_type, $entity_id, $value, array $export_options) {
    if ($export_options['entity_reference_format'] == 'link') {
      if ($entity = entity_load($entity_type, $entity_id)) {
        $record[] = $entity->label();
        $record[] = $entity->id();
        $record[] = $entity->toUrl('canonical', ['absolute' => TRUE])
          ->toString();
      }
      else {
        $record[] = t('Unable to load @entity_type:@entity_id', ['@entity_type' => $entity_type, '@entity_id' => $entity_id]);
        $record[] = $value;
        $record[] = '';
      }
    }
    else {
      $record[] = $value;
    }
  }

  /**
   * Format entity header.
   *
   * @param array $header
   *   Export header.
   * @param string $title
   *   Column title.
   * @param string $name
   *   Column name.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatHeaderEntity(array &$header, $title, $name, array $export_options) {
    if ($export_options['entity_reference_format'] == 'link') {
      if ($export_options['header_keys'] == 'label') {
        $header[] = $title . ' Title';
        $header[] = $title . ' ID';
        $header[] = $title . ' URL';
      }
      else {
        $header[] = $name . '_title';
        $header[] = $name . '_id';
        $header[] = $name . '_url';
      }
    }
    else {
      $header[] = ($export_options['header_keys'] == 'label') ? $title : $name;
    }
  }

}
